#ifndef CNGGGPS_GRAMMAR
#define CNGGGPS_GRAMMAR

#include <string>
#include <vector>
#include "Result.hpp"
#include "Context.hpp"

/**
 * \brief Grammar Abstract Class
 * \author Stephen E. Baker
 * \date 2006-2007
 * \version 1.0
 *
 * Abstract class which is inherited by all symbols in the grammar, and 
 * indirectly by all functions.  This class defines basic methods for
 * interaction between symbols and the rest of the GP system.
 */
class Grammar
{
public:
	Grammar();

	/**
	 * Returns a string reprsenting the particular grammar symbol
	 * the object represents.
	 */
	std::string getGrammarID() const;

	/**
	 * Executes the symbol.  In the case of non-functions this method calls
	 * the appropriate child arguments, coordinating resulting values.  In
	 * functions this method performs the necessary computation.
	 *
	 * \param[in,out] context
	 * 	The context or key-value pairs to be passed to the functions
	 * \return
	 * 	The result of the primary argument
	 */
	virtual Result* execute(Context &context)=0;

	/**
	 * Determines how this symbol and its children are represented when
	 * a string is used to represent the program.
	 *
	 * \param[in] context
	 * 	Unused currently, a reimplementation may use the context to
	 * 	determine actual/formal parameter pairs or to print out
	 * 	global variables
	 *
	 * \return
	 * 	A string representing this node and all of its arguments
	 */
	virtual std::string toString(Context &context)=0;

	/**
	 * Sets the child nodes of this grammar.  These nodes are typically
	 * created by GrammarRules to match the given grammar.
	 *
	 * \param[in] pvpgramExpression
	 * 	A vector of symbols which follow from this symbol
	 * \param[in] iRule
	 * 	The corresponding rule number from GrammarRules.  May be useful
	 * 	if different rules must be put together differently, for
	 * 	example if a symbol may either be followed by a function and
	 * 	two terminals or by a single terminal then the object must
	 * 	know which it currently must handle.
	 */
	void setExpression(std::vector<Grammar*>* pvpgramExpression, int iRule);
	
	/**
	 * Returns the children of this node.
	 * \param[out] pvpgramExpression
	 * 	A vector of symbols which follow from this symbol, NULL if there
	 * 	is none.
	 * \return
	 * 	The corresponding rule number from GrammarRules, -1 if there is
	 * 	none.
	 */
	int getExpression(std::vector<Grammar*>* &pvpgramExpression) const;
	
	/**
	 * Returns the number of children this node has.
	 */
	int getNumArguments() const;
	
	/**
	 * Returns a pointer to a particular child of this node.
	 *
	 * \param[in] iArgument
	 * 	The child to be returned
	 */
	Grammar* getArgument(int iArgument) const;

	/**
	 * Removes this nodes children and their subtrees, freeing the
	 * corresponding memory.
	 */
	void deleteExpression();

	/**
	 * Returns the depth of this particular node in the symbol tree.
	 */
	int getDepth() const;

	/**
	 * Sets the depth of this node in the symbol tree.
	 *
	 * \param[in] iDepth
	 * 	The depth
	 */
	void setDepth(int iDepth);

	/**
	 * Creates a subtree identical to this one in new memory and returns
	 * a pointer to it.
	 */
	virtual Grammar* clone()=0;

	virtual	~Grammar();

protected:
	int iRule;
	std::vector<Grammar*>* pvpgramExpression;
	std::string strGrammarID;
	int iDepth;

	/**
	 * Performs common operations for cloning symbols.  Specifically this
	 * method copies the children nodes and the depth.
	 *
	 * \param[in,out] pgramClone
	 * 	The clone node being produced
	 */
	void clone(Grammar* pgramClone);
};

#endif

